@@ -93,6 +93,9 @@ not supported by the remote, no hash will be returned.  With the
 download flag, the file will be downloaded from the remote and
 hashed locally enabling any hash for any remote.
 
+For the MD5 and SHA1 algorithms there are also dedicated commands,
+[md5sum](/commands/rclone_md5sum/) and [sha1sum](/commands/rclone_sha1sum/).
+
 This command can also hash data received on standard input (stdin),
 by not passing a remote:path, or by passing a hyphen as remote:path
 when there is data to read (if not, the hypen will be treated literaly,
