@@ -18,10 +18,10 @@ var commandDefinition = &cobra.Command{
 	Long: `
 This removes empty directory given by path. Will not remove the path if it
 has any objects in it, not even empty subdirectories. Use
-command ` + "`rmdirs`" + ` (or ` + "`delete`" + ` with option ` + "`--rmdirs`" + `)
-to do that.
+command [rmdirs](/commands/rclone_rmdirs/) (or [delete](/commands/rclone_delete/)
+with option ` + "`--rmdirs`" + `) to do that.
 
-To delete a path and any objects in it, use ` + "`purge`" + ` command.
+To delete a path and any objects in it, use [purge](/commands/rclone_purge/) command.
 `,
 	Run: func(command *cobra.Command, args []string) {
 		cmd.CheckArgs(1, 1, command, args)
