@@ -40,7 +40,7 @@ func (l *CacheMode) Set(s string) error {
 			return nil
 		}
 	}
-	return fmt.Errorf("Unknown cache mode level %q", s)
+	return fmt.Errorf("unknown cache mode level %q", s)
 }
 
 // Type of the value
@@ -52,7 +52,7 @@ func (l *CacheMode) Type() string {
 func (l *CacheMode) UnmarshalJSON(in []byte) error {
 	return fs.UnmarshalJSONFlag(in, l, func(i int64) error {
 		if i < 0 || i >= int64(len(cacheModeToString)) {
-			return fmt.Errorf("Unknown cache mode level %d", i)
+			return fmt.Errorf("unknown cache mode level %d", i)
 		}
 		*l = CacheMode(i)
 		return nil
